import os
import typer
from rich import print, box
from rich.prompt import Prompt, IntPrompt, Confirm
from rich.table import Table
from pathlib import Path
from contextlib import contextmanager

from summarize_gutenberg.get_books import fetch_default_books, process_books
from summarize_gutenberg.get_text import write_text_to_file
from summarize_gutenberg.make_summary import save_summary, print_summary
from summarize_gutenberg.api import Book, BooksDB

FILE_DIR = Path("./files/")
SUMMARY_DIR = FILE_DIR / "summaries"

app = typer.Typer()



def get_default_books():
    """
    Get the default book info then populate the database with corresponding Book objects
    """
    print("\n[italic yellow]Retrieving default book data...")
    books = process_books(fetch_default_books())
    for book in books:
        with books_db() as db:
            db.add_book(Book.from_dict(books[book]))
    
@app.command()
def default():
    """
    Default behavior when given no CLI options: get the 32 most popular books and guide the user interactively through the summarization process.
    """
    with books_db() as db:
        book_count = db.count()
        if book_count:  # If there are any books in the db at this point they are leftover from previous incomplete executions
            db.delete_all()

    get_default_books()

    table = Table(box=box.SQUARE_DOUBLE_HEAD, border_style="magenta")
    table.add_column('No.')
    table.add_column('[bold cyan]Title', max_width=75, no_wrap=False)
    table.add_column('[bold magenta]Author')
    table.add_column('[bold yellow]Fulltext URL')

    with books_db() as db:
        books = db.list_books()
        for order_num, book in enumerate(books, start=1):
            table.add_row(f'{str(order_num)}.', book.title, book.author, f"[yellow]{book.url}")
            order_num += 1
    print('\n')
    print(table)
    print('\n')

    max_choice = len(books)
    choice = Prompt.ask("Select a book by number")
    while not choice.isdigit() or int(choice) < 1 or int(choice) > max_choice:
        choice = Prompt.ask("[red]Please choose a number between 1 and 32")

    selected_book = books[int(choice) - 1]
    
    print(f"\nYou have chosen [bold cyan]{selected_book.title}[/bold cyan] by [bold magenta]{selected_book.author}[/bold magenta].")
    filepath = FILE_DIR / Path(selected_book.filename)

    if filepath.exists():
        print(f"The book has previously been saved to {filepath}.")
    else:
        print("[italic yellow]\nRetrieving book text...[/italic yellow]")
        write_text_to_file(selected_book.url, filepath)
        print(f"\nText of {selected_book.title} saved to {filepath}.")

    choice = Prompt.ask("\nDo you want to [P]rint or [S]ave your summary?", choices=['p', 's'])
    chunks = IntPrompt.ask("How many lines per chunk?", default=400)

    # if chunks < 50:
    #     print("[red bold]Warning[/red bold]: choosing a low value could take a lot of time and resources.")
    #     confirmation = Confirm.ask("Are you sure?")
        
    if choice == 'p':
        print_summary(filepath, chunks)
    else:
        target_filepath = SUMMARY_DIR / Path(selected_book.filename)
        save_summary(filepath, target_filepath, chunks)
        print(f'\nSummary saved to {target_filepath}.')

    with books_db() as db:
        db.delete_all()

def get_path():
    db_path_env = os.getenv("BOOKS_DB_DIR", "")
    if db_path_env:
        db_path = Path(db_path_env)
    else:
        db_path = Path(__file__).parent / "books_db"
    return db_path

@contextmanager
def books_db():
    db_path = get_path()
    db = BooksDB(db_path)
    try:
        yield db
    finally:
        db.close()





import pytest
import sys
from pathlib import Path

sys.path.append(str(Path(__file__).parent.parent / "src"))

from summarize_gutenberg.api import Book

@pytest.fixture()
def book_fixture():
    """
    Create a Book.
    """
    book = Book(
        id=1,
        title="Yesterday's Tomorrows",
        author="Wilfred Sinecure",
        url="https://www.gutenberg.org/",
        filename="yesterdaystomorrows.txt",
    )

    return book


"""
This module contains functions for fetching the first page of English-language results from the Gutendex API and creating a dictionary for each book,
in which the key is an integer representing the book's order in printed results and the value is a dictionary containing relevant book information.
Author names are converted to First Middle Last format.
"""

import requests


def remove_parens(author):
    """
    Remove parentheticals from an author's name, for passing back to author_parse function
    """
    author = author.split()
    for name in author:
        if name[0] == "(":
            total_parens = len(author) - author.index(name)
    return " ".join(author[:-total_parens])


def author_parse(author):
    """
    Take the author information which is stored in Last, First Middle format and convert it to First Middle Last.
    For now it only handles the first author, so second authors of multiauthor books are out of luck.
    """
    if not author:
        return ""

    if author[-1] == ")":
        author = remove_parens(author)

    split_author = author.split(",")

    if len(split_author) == 1:  # One-name authors require no processing
        return author
    elif (
        len(split_author) == 2
    ):  # Presence of only one comma implies 'conventional' name structure (last + first [+ middle])
        return split_author[1].lstrip() + " " + split_author[0]
    else:  # Presence of more than one comma implies a suffix like "Jr."
        return split_author[1].lstrip() + " " + split_author[0] + " " + split_author[2].lstrip()


def author_check(authors):
    """
    Find and return a book's (first) author.
    """
    if not authors:
        return "No author found."

    try:
        name_dict = authors[0]
    except KeyError as e:
        raise KeyError(f"'authors' in unexpected format (should be list)\nOriginal error: {e}")

    try:
        author = name_dict["name"]
    except KeyError as e:
        raise KeyError(f"No field named 'name' in authors\nOriginal error: {e}")

    return author


def url_check(formats):
    """
    Find and return a book's full-text url.
    """
    if not formats:
        return "No URLs found."

    try:
        url = formats["text/plain; charset=us-ascii"]
    except KeyError as e:
        raise KeyError(f"No plaintext URL or plaintext key format has changed\nOriginal error: {e}")

    return url

def create_filename(title):
    """
    Create a filename for the book from a shortened version of its title.
    """
    res_list = []
    colons = {':', ';'}
    puncts = {',', ' ', '.', '—', '-', "'", '"'}

    for char in title:
        if char in colons:
            break
        elif char in puncts:
            continue
        else:
            res_list.append(char.lower())
    
    res_list.append('.txt')

    return ''.join(res_list)    

def fetch_default_books():
    """
    Download info of the 32 most popular books on Project Gutenberg and return a json of them.
    """
    base_url = "https://gutendex.com/books?languages=en"

    try:
        book_request = requests.get(base_url, params={"q": "requests+lang:en"})
        book_request.raise_for_status()
    except requests.exceptions.HTTPError as errh:
        print("Http Error:", errh)
        raise
    except requests.exceptions.ConnectionError as errc:
        print("Connection error:", errc)
        raise
    except requests.exceptions.Timeout as errt:
        print("Timeout error:", errt)
        raise
    except requests.exceptions.RequestException as err:
        print("Exotic error:", err)
        raise

    books = book_request.json()["results"]

    return books


def process_books(books):
    """ 
    Create a dictionary of fetched books where the key is a sequential number and the value is a dictionary of book info.
    """
    book_data = {}
    book_num = 1
    for book in books:
        title = book.get("title", None)

        if title:
            filename = create_filename(title)
        else:
            filename = None

        authors = book.get("authors")
        author = author_parse(author_check(authors))

        formats = book.get("formats")
        url = url_check(formats)

        book_data[book_num] = {"title": title, "author": author, "url": url, "filename": filename}

        book_num += 1

    return book_data

if __name__ == "__main__":
    books = process_books(fetch_default_books()) 

    book_list = [book for book in books]
    for book in book_list:
        print(books[book]['filename'])



import pytest
from pathlib import Path
from tempfile import TemporaryDirectory

from summarize_gutenberg.api import Book, BooksDB

@pytest.fixture()
def books_db():
    with TemporaryDirectory() as db_dir:
        db_path = Path(db_dir)
        db = BooksDB(db_path)
        yield db
        db.close()

def test_empty_db(books_db):
    assert books_db.count() == 0

def test_add_one_book(books_db):
    books_db.add_book(Book())

    assert books_db.count() == 1

def test_add_two_books(books_db):
    books_db.add_book(Book())
    books_db.add_book(Book())

    assert books_db.count() == 2

def test_delete_book(books_db):
    books_db.add_book(Book(id=1))
    books_db.add_book(Book(id=2))
    books_db.delete_book(2)

    assert books_db.count() == 1

def test_delete_all(books_db):
    books_db.add_book(Book(id=1))
    books_db.add_book(Book(id=2))
    books_db.delete_all()

    assert books_db.count() == 0

def test_get_book(books_db, book_fixture):
    book = book_fixture
    books_db.add_book(book)
    gotten_book = books_db.get_book(1)

    assert book == gotten_book

def test_list_books(books_db):
    books_db.add_book(Book(id=1))
    books_db.add_book(Book(id=2))
    books_db.add_book(Book(id=3))
    listed_books = books_db.list_books()
    expected_ids = {1, 2, 3}

    assert len(listed_books) == 3
    for book in listed_books:
        assert book.id in expected_ids

def test_update_book(books_db, book_fixture):
    book = book_fixture
    books_db.add_book(book)
    new_data = {
        "title": "Tomorrow's Yesterdays",
        "author": "Clifton Semaphore",
        "url": "https://www.csemaphore.com/2",
        "filename": "tomorrowsyesterdays.txt",
    }

    new_book = Book.from_dict(new_data)
    books_db.update_book(book.id, new_book)
    updated_book_data = books_db.get_book(book.id).to_dict()

    assert updated_book_data['id'] == book.id
    assert updated_book_data['title'] == new_data['title']
    assert updated_book_data['author'] == new_data['author']
    assert updated_book_data['url'] == new_data['url']
    assert updated_book_data['filename'] == new_data['filename']







"""
The content of this module was adapted and mostly taken verbatim from the gutenbergypy project:
https://github.com/raduangelescu/gutenbergpy/blob/master/gutenbergpy/textget.py
which was itself taken from the gutenberg project:
https://github.com/c-w/Gutenberg
in which the strip_headers function was ported from a C++ version by Johannes Krugel which no longer appears to be available.
I have added a line to TEXT_END_MARKERS and changed the return value of strip_headers to encode the output text
"""

import os
import requests


TEXT_START_MARKERS = frozenset(
    (
        "*END*THE SMALL PRINT",
        "*** START OF THE PROJECT GUTENBERG",
        "*** START OF THIS PROJECT GUTENBERG",
        "This etext was prepared by",
        "E-text prepared by",
        "Produced by",
        "Distributed Proofreading Team",
        "Proofreading Team at http://www.pgdp.net",
        "http://gallica.bnf.fr)",
        "      http://archive.org/details/",
        "http://www.pgdp.net",
        "by The Internet Archive)",
        "by The Internet Archive/Canadian Libraries",
        "by The Internet Archive/American Libraries",
        "public domain material from the Internet Archive",
        "Internet Archive)",
        "Internet Archive/Canadian Libraries",
        "Internet Archive/American Libraries",
        "material from the Google Print project",
        "*END THE SMALL PRINT",
        "***START OF THE PROJECT GUTENBERG",
        "This etext was produced by",
        "*** START OF THE COPYRIGHTED",
        "The Project Gutenberg",
        "http://gutenberg.spiegel.de/ erreichbar.",
        "Project Runeberg publishes",
        "Beginning of this Project Gutenberg",
        "Project Gutenberg Online Distributed",
        "Gutenberg Online Distributed",
        "the Project Gutenberg Online Distributed",
        "Project Gutenberg TEI",
        "This eBook was prepared by",
        "http://gutenberg2000.de erreichbar.",
        "This Etext was prepared by",
        "This Project Gutenberg Etext was prepared by",
        "Gutenberg Distributed Proofreaders",
        "Project Gutenberg Distributed Proofreaders",
        "the Project Gutenberg Online Distributed Proofreading Team",
        "**The Project Gutenberg",
        "*SMALL PRINT!",
        "More information about this book is at the top of this file.",
        "tells you about restrictions in how the file may be used.",
        "l'authorization à les utilizer pour preparer ce texte.",
        "of the etext through OCR.",
        "*****These eBooks Were Prepared By Thousands of Volunteers!*****",
        "We need your donations more than ever!",
        " *** START OF THIS PROJECT GUTENBERG",
        "****     SMALL PRINT!",
        '["Small Print" V.',
        "      (http://www.ibiblio.org/gutenberg/",
        "and the Project Gutenberg Online Distributed Proofreading Team",
        "Mary Meehan, and the Project Gutenberg Online Distributed Proofreading",
        "                this Project Gutenberg edition.",
    )
)

TEXT_END_MARKERS = frozenset(
    (
        "*** END OF THE PROJECT GUTENBERG",
        "*** END OF THIS PROJECT GUTENBERG",
        "***END OF THE PROJECT GUTENBERG",
        "End of the Project Gutenberg",
        "End of The Project Gutenberg",
        "Ende dieses Project Gutenberg",
        "by Project Gutenberg",
        "End of Project Gutenberg",
        "End of this Project Gutenberg",
        "Ende dieses Projekt Gutenberg",
        "        ***END OF THE PROJECT GUTENBERG",
        "*** END OF THE COPYRIGHTED",
        "End of this is COPYRIGHTED",
        "Ende dieses Etextes ",
        "Ende dieses Project Gutenber",
        "Ende diese Project Gutenberg",
        "**This is a COPYRIGHTED Project Gutenberg Etext, Details Above**",
        "Fin de Project Gutenberg",
        "The Project Gutenberg Etext of ",
        "Ce document fut presente en lecture",
        "Ce document fut présenté en lecture",
        "More information about this book is at the top of this file.",
        "We need your donations more than ever!",
        "END OF PROJECT GUTENBERG",
        " End of the Project Gutenberg",
        " *** END OF THIS PROJECT GUTENBERG",
        "            *** END OF THE PROJECT GUTENBERG",
    )
)

LEGALESE_START_MARKERS = frozenset(("<<THIS ELECTRONIC VERSION OF",))
LEGALESE_END_MARKERS = frozenset(("SERVICE THAT CHARGES FOR DOWNLOAD",))


def is_valid_utf8(byte_sequence):
    try:
        byte_sequence.encode("utf-8")
        return True
    except UnicodeDecodeError:
        return False

def strip_headers(text):
    lines = text.splitlines()
    sep = os.linesep
    sep = sep.encode("utf-8")
    out = []
    i = 0
    footer_found = False
    ignore_section = False

    for line in lines:
        reset = False

        if i <= 600:
            # Check if the header ends here
            if any(line.startswith(token.encode("utf-8")) for token in TEXT_START_MARKERS):
                reset = True

            # If it's the end of the header, delete the output produced so far.
            # May be done several times, if multiple lines occur indicating the
            # end of the header
            if reset:
                out = []
                continue

        if i >= 100:
            # Check if the footer begins here
            if any(line.startswith(token.encode("utf-8")) for token in TEXT_END_MARKERS):
                footer_found = True

            # If it's the beginning of the footer, stop output
            if footer_found:
                break

        if any(line.startswith(token.encode("utf-8")) for token in LEGALESE_START_MARKERS):
            ignore_section = True
            continue
        elif any(line.startswith(token.encode("utf-8")) for token in LEGALESE_END_MARKERS):
            ignore_section = False
            continue

        if not ignore_section:
            stripline = line.rstrip(sep)
            out.append(stripline)
            i += 1

    return str(sep.join(out), encoding="utf-8")

def write_text_to_file(url, file_path):
    text_request = requests.get(url, stream=True)

    if text_request.status_code != 200:
        raise Exception("Book not found...")

    text_content = text_request.content
    cleaned_text = strip_headers(text_content)

    with open(file_path, "w", encoding='utf-8') as file:
        file.write(cleaned_text)

    return file_path


import pytest
import requests

from summarize_gutenberg.get_books import fetch_default_books, process_books, author_check, url_check

def test_gutendex_api():
    base_url = 'https://gutendex.com/books?languages=en'
    book_request = requests.get(base_url, params={'q': 'requests+lang:en'})
    assert book_request.status_code == 200
    assert 'results' in book_request.json()
    assert len(book_request.json()['results']) == 32

def test_fetch_default_books_success(mocker):
    mock_response = mocker.Mock()
    mock_response.json.return_value = {'results': ['book1', 'book2']}
    mock_response.raise_for_status.return_value = None
    mocker.patch('requests.get', return_value=mock_response)

    books = fetch_default_books()
    assert books == ['book1', 'book2']

def test_fetch_default_books_http_error(mocker):
    mocker.patch('requests.get', side_effect=requests.exceptions.HTTPError("Http Error"))

    with pytest.raises(requests.exceptions.HTTPError):
        fetch_default_books()

def test_fetch_default_books_connection_error(mocker):
    mocker.patch('requests.get', side_effect=requests.exceptions.ConnectionError("Connection Error"))

    with pytest.raises(requests.exceptions.ConnectionError):
        fetch_default_books()

def test_fetch_default_books_timeout(mocker):
    mocker.patch('requests.get', side_effect=requests.exceptions.Timeout("Timeout Error"))

    with pytest.raises(requests.exceptions.Timeout):
        fetch_default_books()

def test_fetch_default_books_request_exception(mocker):
    mocker.patch('requests.get', side_effect=requests.exceptions.RequestException("Exotic Error"))

    with pytest.raises(requests.exceptions.RequestException):
        fetch_default_books()

test_authors_no_list = {
    'name': 'Doe, John'
}

test_authors_no_name = [
    {
    'mame': 'Doe, John'
    }
    ]

def test_author_check_no_list():
    with pytest.raises(KeyError) as excinfo:
        author_check(test_authors_no_list)
    assert "'authors' in unexpected format (should be list)" in str(excinfo.value)

def test_author_check_no_name():
    with pytest.raises(KeyError) as excinfo:
        author_check(test_authors_no_name)
    assert "No field named 'name' in authors" in str(excinfo.value)

test_formats_no_plaintext = {
    'wrong_format': 'book_url',
    }

def test_url_check_no_plaintext():
    with pytest.raises(KeyError) as excinfo:
        url_check(test_formats_no_plaintext)
    assert "No plaintext URL or plaintext key format has changed" in str(excinfo.value)

book_data = [
    # Normal data
    (
        {
            'id': 1,
            'title': 'Sample Book',
            'authors': [{'name': 'Doe, John'}],
            'formats': {'text/plain; charset=us-ascii': 'http://example.com'}
        },
        {
            1: {
                'title': 'Sample Book',
                'author': 'John Doe',
                'url': 'http://example.com',
                'filename': 'samplebook.txt',
            }
        }
    ),
    # No title
    (
        {
            'id': 1,
            'authors': [{'name': 'Doe, John'}],
            'formats': {'text/plain; charset=us-ascii': 'http://example.com'}
        },
        {
            1: {
                'title': None,
                'author': 'John Doe',
                'url': 'http://example.com',
                'filename': None,
            }
        }
    ),
    # No author
    (
        {
            'id': 1,
            'title': 'Sample Book',
            'formats': {'text/plain; charset=us-ascii': 'http://example.com'}
        },
        {
            1: {
                'title': 'Sample Book',
                'author': 'No author found.',
                'url': 'http://example.com',
                'filename': 'samplebook.txt',
            }
        }
    ),
    # No formats
    (
        {
            'id': 1,
            'title': 'Sample Book',
            'authors': [{'name': 'Doe, John'}]
        },
        {
            1: {
                'title': 'Sample Book',
                'author': 'John Doe',
                'url': 'No URLs found.',
                'filename': 'samplebook.txt',
            }
        }
    ),
    # No title or author
    (
        {
            'id': 1,
            'formats': {'text/plain; charset=us-ascii': 'http://example.com'}
        },
        {
            1: {
                'title': None,
                'author': 'No author found.',
                'url': 'http://example.com',
                'filename': None,
            }
        }
    ),
    # no title or formats
    (
        {
            'id': 1,
            'authors': [{'name': 'Doe, John'}],
        },
        {
            1: {
                'title': None,
                'author': 'John Doe',
                'url': 'No URLs found.',
                'filename': None,
            }
        }
    ),
    # no author or formats
    (
        {
            'id': 1,
            'title': 'Sample Book',
        },
        {
            1: {
                'title': 'Sample Book',
                'author': 'No author found.',
                'url': 'No URLs found.',
                'filename': 'samplebook.txt',
            }
        }
    ),
    # no title author or formats
    (
        {
            'id': 1,
        },
        {
            1: {
                'title': None,
                'author': 'No author found.',
                'url': 'No URLs found.',
                'filename': None,
            }
        }
    ),
]


@pytest.mark.parametrize('input, expected', book_data)
def test_process_books(input, expected):
    result = process_books([input])
    assert result == expected, f'Expected {expected}, but got {result}'


from transformers import AutoTokenizer, AutoModelForSeq2SeqLM

"""
This script is just to show that the selected summarization technology works. Next step is to try with longer texts.
Running it the first time takes awhile.
Swap in any of the four texts provided in inputs to see the different summaries.
"""

# From the introduction to "The Ethics of Aristotle"
text1 = """
The _Ethics_ of Aristotle is one half of a single treatise of which his
_Politics_ is the other half. Both deal with one and the same subject.
This subject is what Aristotle calls in one place the “philosophy of
human affairs;” but more frequently Political or Social Science. In the
two works taken together we have their author’s whole theory of human
conduct or practical activity, that is, of all human activity which is
not directed merely to knowledge or truth. The two parts of this
treatise are mutually complementary, but in a literary sense each is
independent and self-contained. The proem to the _Ethics_ is an
introduction to the whole subject, not merely to the first part; the
last chapter of the _Ethics_ points forward to the _Politics_, and
sketches for that part of the treatise the order of enquiry to be
pursued (an order which in the actual treatise is not adhered to).

The principle of distribution of the subject-matter between the two
works is far from obvious, and has been much debated. Not much can be
gathered from their titles, which in any case were not given to them by
their author. Nor do these titles suggest any very compact unity in the
works to which they are applied: the plural forms, which survive so
oddly in English (Ethic_s_, Politic_s_), were intended to indicate the
treatment within a single work of a _group_ of connected questions. The
unity of the first group arises from their centring round the topic of
character, that of the second from their connection with the existence
and life of the city or state. We have thus to regard the _Ethics_ as
dealing with one group of problems and the _Politics_ with a second,
both falling within the wide compass of Political Science. Each of
these groups falls into sub-groups which roughly correspond to the
several books in each work. The tendency to take up one by one the
various problems which had suggested themselves in the wide field
obscures both the unity of the subject-matter and its proper
articulation. But it is to be remembered that what is offered us is
avowedly rather an enquiry than an exposition of hard and fast
doctrine.
"""

# The opening of Milton's Paradise Lost
text2 = """
Of Man’s first disobedience, and the fruit
Of that forbidden tree whose mortal taste
Brought death into the World, and all our woe,
With loss of Eden, till one greater Man
Restore us, and regain the blissful seat,
Sing, Heavenly Muse, that, on the secret top
Of Oreb, or of Sinai, didst inspire
That shepherd who first taught the chosen seed
In the beginning how the heavens and earth
Rose out of Chaos: or, if Sion hill
Delight thee more, and Siloa’s brook that flowed
Fast by the oracle of God, I thence
Invoke thy aid to my adventurous song,
That with no middle flight intends to soar
Above th’ Aonian mount, while it pursues
Things unattempted yet in prose or rhyme.
And chiefly thou, O Spirit, that dost prefer
Before all temples th’ upright heart and pure,
Instruct me, for thou know’st; thou from the first
Wast present, and, with mighty wings outspread,
Dove-like sat’st brooding on the vast Abyss,
And mad’st it pregnant: what in me is dark
Illumine, what is low raise and support;
That, to the height of this great argument,
I may assert Eternal Providence,
And justify the ways of God to men.
"""

# The first few paragraphs of Augustine's Confessions
text3 = """
Great art Thou, O Lord, and greatly to be praised; great is Thy power,
and Thy wisdom infinite. And Thee would man praise; man, but a particle
of Thy creation; man, that bears about him his mortality, the witness of
his sin, the witness that Thou resistest the proud: yet would man praise
Thee; he, but a particle of Thy creation. Thou awakest us to delight in
Thy praise; for Thou madest us for Thyself, and our heart is restless,
until it repose in Thee. Grant me, Lord, to know and understand which is
first, to call on Thee or to praise Thee? and, again, to know Thee or
to call on Thee? for who can call on Thee, not knowing Thee? for he that
knoweth Thee not, may call on Thee as other than Thou art. Or, is it
rather, that we call on Thee that we may know Thee? but how shall they
call on Him in whom they have not believed? or how shall they believe
without a preacher? and they that seek the Lord shall praise Him: for
they that seek shall find Him, and they that find shall praise Him.
I will seek Thee, Lord, by calling on Thee; and will call on Thee,
believing in Thee; for to us hast Thou been preached. My faith, Lord,
shall call on Thee, which Thou hast given me, wherewith Thou hast
inspired me, through the Incarnation of Thy Son, through the ministry of
the Preacher.

And how shall I call upon my God, my God and Lord, since, when I call
for Him, I shall be calling Him to myself? and what room is there within
me, whither my God can come into me? whither can God come into me, God
who made heaven and earth? is there, indeed, O Lord my God, aught in me
that can contain Thee? do then heaven and earth, which Thou hast made,
and wherein Thou hast made me, contain Thee? or, because nothing which
exists could exist without Thee, doth therefore whatever exists contain
Thee? Since, then, I too exist, why do I seek that Thou shouldest enter
into me, who were not, wert Thou not in me? Why? because I am not gone
down in hell, and yet Thou art there also. For if I go down into hell,
Thou art there. I could not be then, O my God, could not be at all,
wert Thou not in me; or, rather, unless I were in Thee, of whom are all
things, by whom are all things, in whom are all things? Even so, Lord,
even so. Whither do I call Thee, since I am in Thee? or whence canst
Thou enter into me? for whither can I go beyond heaven and earth, that
thence my God should come into me, who hath said, I fill the heaven and
the earth.

Do the heaven and earth then contain Thee, since Thou fillest them? or
dost Thou fill them and yet overflow, since they do not contain Thee?
And whither, when the heaven and the earth are filled, pourest Thou
forth the remainder of Thyself? or hast Thou no need that aught contain
Thee, who containest all things, since what Thou fillest Thou fillest
by containing it? for the vessels which Thou fillest uphold Thee not,
since, though they were broken, Thou wert not poured out. And when Thou
art poured out on us, Thou art not cast down, but Thou upliftest us;
Thou art not dissipated, but Thou gatherest us. But Thou who fillest
all things, fillest Thou them with Thy whole self? or, since all things
cannot contain Thee wholly, do they contain part of Thee? and all at
once the same part? or each its own part, the greater more, the smaller
less? And is, then one part of Thee greater, another less? or, art Thou
wholly every where, while nothing contains Thee wholly?
"""

# The first few paragraphs of the Preface to the First Edition of Kant's Critique of Pure Reason
text4 = """
Human reason, in one sphere of its cognition, is called upon to
consider questions, which it cannot decline, as they are presented by
its own nature, but which it cannot answer, as they transcend every
faculty of the mind.

It falls into this difficulty without any fault of its own. It begins
with principles, which cannot be dispensed with in the field of
experience, and the truth and sufficiency of which are, at the same
time, insured by experience. With these principles it rises, in
obedience to the laws of its own nature, to ever higher and more remote
conditions. But it quickly discovers that, in this way, its labours
must remain ever incomplete, because new questions never cease to
present themselves; and thus it finds itself compelled to have recourse
to principles which transcend the region of experience, while they are
regarded by common sense without distrust. It thus falls into confusion
and contradictions, from which it conjectures the presence of latent
errors, which, however, it is unable to discover, because the
principles it employs, transcending the limits of experience, cannot be
tested by that criterion. The arena of these endless contests is called
_Metaphysic_.

Time was, when she was the _queen_ of all the sciences; and, if we take
the will for the deed, she certainly deserves, so far as regards the
high importance of her object-matter, this title of honour. Now, it is
the fashion of the time to heap contempt and scorn upon her; and the
matron mourns, forlorn and forsaken, like Hecuba:

Modo maxima rerum,
Tot generis, natisque potens...
Nunc trahor exul, inops.
—Ovid, Metamorphoses. xiii


At first, her government, under the administration of the _dogmatists_,
was an absolute _despotism_. But, as the legislative continued to show
traces of the ancient barbaric rule, her empire gradually broke up, and
intestine wars introduced the reign of _anarchy;_ while the _sceptics_,
like nomadic tribes, who hate a permanent habitation and settled mode
of living, attacked from time to time those who had organized
themselves into civil communities. But their number was, very happily,
small; and thus they could not entirely put a stop to the exertions of
those who persisted in raising new edifices, although on no settled or
uniform plan. In recent times the hope dawned upon us of seeing those
disputes settled, and the legitimacy of her claims established by a
kind of _physiology_ of the human understanding—that of the celebrated
Locke. But it was found that—although it was affirmed that this
so-called queen could not refer her descent to any higher source than
that of common experience, a circumstance which necessarily brought
suspicion on her claims—as this _genealogy_ was incorrect, she
persisted in the advancement of her claims to sovereignty. Thus
metaphysics necessarily fell back into the antiquated and rotten
constitution of _dogmatism_, and again became obnoxious to the contempt
from which efforts had been made to save it. At present, as all
methods, according to the general persuasion, have been tried in vain,
there reigns nought but weariness and complete _indifferentism_—the
mother of chaos and night in the scientific world, but at the same time
the source of, or at least the prelude to, the re-creation and
reinstallation of a science, when it has fallen into confusion,
obscurity, and disuse from ill directed effort.

For it is in reality vain to profess _indifference_ in regard to such
inquiries, the object of which cannot be indifferent to humanity.
Besides, these pretended _indifferentists_, however much they may try
to disguise themselves by the assumption of a popular style and by
changes on the language of the schools, unavoidably fall into
metaphysical declarations and propositions, which they profess to
regard with so much contempt. At the same time, this indifference,
which has arisen in the world of science, and which relates to that
kind of knowledge which we should wish to see destroyed the last, is a
phenomenon that well deserves our attention and reflection. It is
plainly not the effect of the levity, but of the matured _judgement_[1]
of the age, which refuses to be any longer entertained with illusory
knowledge, It is, in fact, a call to reason, again to undertake the
most laborious of all tasks—that of self-examination, and to establish
a tribunal, which may secure it in its well-grounded claims, while it
pronounces against all baseless assumptions and pretensions, not in an
arbitrary manner, but according to its own eternal and unchangeable
laws. This tribunal is nothing less than the _Critical Investigation of
Pure Reason_.
"""

tokenizer = AutoTokenizer.from_pretrained("pszemraj/pegasus-x-large-book-summary")
model = AutoModelForSeq2SeqLM.from_pretrained("pszemraj/pegasus-x-large-book-summary")

inputs = tokenizer.encode(text2, return_tensors="pt", truncation=True)

summary_ids = model.generate(inputs)
summary = tokenizer.decode(summary_ids[0], skip_special_tokens=True)

print(summary)



import pytest
from summarize_gutenberg.get_books import create_filename

titles = [
    ('Dracula', 'dracula.txt'), # one word
    ('Franny and Zooey', 'frannyandzooey.txt'), # spaces
    ('Frankenstein; Or, The Modern Prometheus', 'frankenstein.txt'), # semicolon
    ('"Left-Wing" Communism: an Infantile Disorder', 'leftwingcommunism.txt'), # colon, quoation marks, hyphen
    ('We Who Are About To...', 'wewhoareaboutto.txt'), # periods
    ("Swann's Way", 'swannsway.txt'), # apostrophe/single quote
    ("My Life — Volume 1", 'mylifevolume1.txt') # em dash which is weird but one of the top gutenberg books has one
]

@pytest.mark.parametrize('input, expected', titles)
def test_create_filename(input, expected):
    assert create_filename(input) == expected, f'Expected {expected}, but got {create_filename(input)}'


from dataclasses import dataclass
from dataclasses import asdict
from dataclasses import field

from summarize_gutenberg.db import DB

@dataclass
class Book:
    id: int = field(default=None)
    title: str = None
    author: str = None
    url: str = None
    filename: str = None

    @classmethod
    def from_dict(cls, d):
        return Book(**d)
    def to_dict(self):
        return asdict(self)



class BooksDB:
    def __init__(self, db_path):
        self._db_path = db_path
        self._db = DB(db_path, ".books_db")

    def add_book(self, book: Book) -> int:
        """Add a book, return its id."""
        id = self._db.create(book.to_dict())
        self._db.update(id, {"id": id})
        return id

    def get_book(self, book_id: int) -> Book:
        """Return a book with a matching id."""
        db_item = self._db.read(book_id)
        if db_item is not None:
            return Book.from_dict(db_item)
        # else:
            # raise InvalidBookId(book_id)

    def list_books(self):
        """Return a list of books."""
        all = self._db.read_all()
        return [Book.from_dict(t) for t in all]

    def count(self) -> int:
        """Return the number of books in db."""
        return self._db.count()

    def update_book(self, book_id: int, book_mods: Book) -> None:
        """Update a book with modifications."""
        # try:
        self._db.update(book_id, book_mods.to_dict())
        # except KeyError as exc:
        #     raise InvalidBookId(book_id) from exc

    def delete_book(self, book_id: int) -> None:
        """Remove a book from db with given book_id."""
        self._db.delete(book_id)
        # except KeyError as exc:
            # raise InvalidBookId(book_id) from exc

    def delete_all(self) -> None:
        """Remove all books from db."""
        self._db.delete_all()

    def close(self):
        self._db.close()

    def path(self):
        return self._db_path


import pytest
from summarize_gutenberg.get_books import author_parse

authors = [
('Aristotle', 'Aristotle'), # single name
('Austen, Jane', 'Jane Austen'), # first & last
('Stevenson, Robert Louis', 'Robert Louis Stevenson'), # first last middle
('Chesterton, G. K. (Gilbert Keith)', 'G. K. Chesterton'), # parenthetical
('H. D. (Hilda Doolittle)', 'H. D.'), # irregular parenthetical
('Tolkien, J. R. R. (John Ronald Reuel)', 'J. R. R. Tolkien'), # parenthetical with three initials
('Von Arnim, Elizabeth', 'Elizabeth Von Arnim'), # von
('Sanchez, Nellie Van de Grift', 'Nellie Van de Grift Sanchez'), # van
('Martinez de la Torre, Rafael', 'Rafael Martinez de la Torre'), # de la
('Cervantes Saavedra, Miguel de', 'Miguel de Cervantes Saavedra'), # de
('Alger, Horatio, Jr.', 'Horatio Alger Jr.'), # jr
(None, '') # none
]

@pytest.mark.parametrize('input, expected', authors)
def test_author_parse(input, expected):
    assert author_parse(input) == expected, f'Expected {expected}, but got {author_parse(input)}'


from transformers import AutoTokenizer, AutoModelForSeq2SeqLM

tokenizer = AutoTokenizer.from_pretrained("pszemraj/pegasus-x-large-book-summary")
model = AutoModelForSeq2SeqLM.from_pretrained("pszemraj/pegasus-x-large-book-summary")

def read_in_chunks(filepath, chunk_size=800):
    with open(filepath, 'r', encoding='utf-8') as file:
            current_chunk = []
            for line in file:
                cleaned_line = line.strip()
                current_chunk.append(cleaned_line)
                if len(current_chunk) == chunk_size:
                    yield current_chunk
                    current_chunk = []  

            if current_chunk:
                yield current_chunk

def make_summary(chunk):
    inputs = tokenizer.encode(chunk, return_tensors="pt", truncation=True)

    summary_ids = model.generate(inputs)
    summary = tokenizer.decode(summary_ids[0], skip_special_tokens=True)

    return summary



def save_summary(source, target, chunk_size):
    with open(target,'w') as target:  
        for chunk in read_in_chunks(source, chunk_size=chunk_size):
            target.write(make_summary(' '.join(chunk)))

def print_summary(source, chunk_size):
    for chunk in read_in_chunks(source, chunk_size=chunk_size):
        print(make_summary(' '.join(chunk)))




from summarize_gutenberg.api import Book

def test_book_field_access(book_fixture):
    book = book_fixture

    assert book.id == 1
    assert book.title == "Yesterday's Tomorrows"
    assert book.author == "Wilfred Sinecure"
    assert book.url == "https://www.gutenberg.org/"
    assert book.filename == "yesterdaystomorrows.txt"

def test_book_defaults():
    book = Book()

    assert book.title is None
    assert book.author is None
    assert book.url is None
    assert book.filename is None

def test_from_dict(book_fixture):
    book1 = book_fixture
    book2_dict = {
        "id": 1,
        "title": "Yesterday's Tomorrows",
        "author": "Wilfred Sinecure",
        "url": "https://www.gutenberg.org/",
        "filename": "yesterdaystomorrows.txt",
    }
    book2 = Book.from_dict(book2_dict)

    assert book1 == book2

def test_to_dict(book_fixture):
    book1 = book_fixture
    book_dict = book1.to_dict()
    expected =  {
        "id": 1,
        "title": "Yesterday's Tomorrows",
        "author": "Wilfred Sinecure",
        "url": "https://www.gutenberg.org/",
        "filename": "yesterdaystomorrows.txt",
    }

    assert book_dict == expected